/*
 *  apcsetup.c  -- configuration parts for apcupsd package
 *
 *  apcupsd.c   -- Simple Daemon to catch power failure signals from a
 *                 BackUPS, BackUPS Pro, or SmartUPS (from APCC).
 *              -- Now SmartMode support for SmartUPS and BackUPS Pro.
 *
 *  Copyright (C) 1996-98 Andre M. Hedrick <hedrick@astro.dyer.vanderbilt.edu>
 *                Vanderbilt University, Dyer Observatory.
 *  All rights reserved.
 *
 */

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include <apc_defines.h>
#include <apc_struct.h>
#include <apc_extern.h>

#undef __DEBUG_NAME
#undef __FLAG_REPORTS
#undef __SETUP_SELFTEST
#undef __SETUP_EXTENDED

#ifdef __SETUP_EXTENDED
/*********************************************************************/
void setup_ups_extended(UPSINFO *ups)
{ char response[32];
  char a;
  response[0] = '\0';
}
#endif /* __SETUP_EXTENDED */

/*********************************************************************/
char *setup_ups_model_V_codes(char *s, UPSINFO *ups)
{ char *cp;

  switch(s[0])
    { case '0': return(cp = "APC Matrix-UPS 3000");
      case '2': return(cp = "APC Smart-UPS 250");
      case '3':
      case '4': return(cp = "APC Smart-UPS 400");
      case '5': return(cp = "APC Matrix-UPS 5000");
      case '6': return(cp = "APC Smart-UPS 600");
      case '7': return(cp = "APC Smart-UPS 900");
      case '8': return(cp = "APC Smart-UPS 1250");
      case '9': return(cp = "APC Smart-UPS 2000");
      case 'A': return(cp = "APC Smart-UPS 1400");
      case 'B': return(cp = "APC Smart-UPS 1000");
      case 'C': return(cp = "APC Smart-UPS 650");
      case 'D': return(cp = "APC Smart-UPS 420");
      case 'E': return(cp = "APC Smart-UPS 280");
      case 'F': return(cp = "APC Smart-UPS 450");
      case 'G': return(cp = "APC Smart-UPS 700");
      case 'H': return(cp = "APC Smart-UPS 700 XL");
      case 'I': return(cp = "APC Smart-UPS 1000");
      case 'J': return(cp = "APC Smart-UPS 1000 XL");
      case 'K': return(cp = "APC Smart-UPS 1400");
      case 'L': return(cp = "APC Smart-UPS 1400 XL");
      case 'M': return(cp = "APC Smart-UPS 2200");
      case 'N': return(cp = "APC Smart-UPS 2200 XL");
      case 'P': return(cp = "APC Smart-UPS 3000");
      case 'O': return(cp = "APC Smart-UPS 5000");
      default:
        break; }
    return(cp = ups->mode.long_name);
}

/*********************************************************************/
char *setup_ups_model_b_codes(char *s, UPSINFO *ups)
{ char *cp;

/*
 * b     New Firmware revison and model ID String in NN.M.D is the format
 *         This returns two numbers and a character "D"
 *           where NN == UPS ID Code.
 *               12 == Back-UPS Pro 650
 *               13 == Back-UPS Pro 1000
 *               52 == Smart-UPS 700
 *               72 == Smart-UPS 1400
 *           where M == Possible Case Style, unknown???
 *               1 == Stand Alone
 *               8 == Rack Mount
 *               9 == Rack Mount
 *           where D == unknown???
 *
 */

  fprintf(stderr, "\n%s: 'b' %s", ups->argvalue, s);
  return(cp = ups->mode.long_name);
}

/*********************************************************************/
/*
 * JHNC-FIXME:
 *
 * First point: AFAICT this is the *only* place in the program
 *     where a UPSINFO.*.long_name is overwritten.
 *
 * Second point: setup_ups_model() is only called *once* AFAICT.
 *     (by setup_extended() by do_serial()).
 *
 * Conclusion: Why not do something along the lines of:
 *
 *     static char response[32];
 *     ...
 *     ups->mode.long_name = response;
 *
 * Then the GENINFO/INTERNALGENINFO problem magically goes away.
 */
void setup_ups_model(UPSINFO *ups)
{ char response[32];
  char *cp;
  unsigned char b;
  int i;
  response[0] = '\0';

  for (i=0;i<4;i++)
    { b=0x01;
      write(ups->fd, &b, 1);
      sleep(1); }
  getline(response, ups);

  if (strlen(response))
    { ups->mode.long_name[0] = '\0';
      sprintf(ups->mode.long_name, "%s", response);
      return; }

  response[0] = '\0';
  strcpy(response, smart_poll('V', ups));

  if (strlen(response))
    { cp = setup_ups_model_V_codes(response, ups);
      if (cp != ups->mode.long_name)
          sprintf(ups->mode.long_name, "%s", cp);
      return; }

  response[0] = '\0';
  strcpy(response, smart_poll('b', ups));

  if (strlen(response))
    fprintf(stderr, "\n%s: 'b' %s", ups->argvalue,
            setup_ups_model_b_codes(response, ups)); }

/*********************************************************************/
void setup_ups_name(UPSINFO *ups)
{ char response[32];
  char name[8];
  char a;
  int i, j;
  response[0] = '\0';
  name[0] = '\0';

  strncpy(name, ups->name, strlen(ups->name));
  j = strlen(name);
  if (j > 8) j = 8;
  sleep(1);
  a = 'c';
  write(ups->fd, &a, 1);
  sleep(1);
  a = '-';
  write(ups->fd, &a, 1);
  sleep(1);
  a = '\n';
  write(ups->fd, &a, 1);
  sleep(1);

  for (i=0;i<j;i++)
    { a = name[i];
      write(ups->fd, &a, 1);
      sleep(1); }

  if (j < 8)
    { for (i=j;i<8;i++)
        { a = ' ';
          write(ups->fd, &a, 1);
          sleep(1); } }

  sleep(1);
  getline(response, ups);
  sleep(1);
  getline(response, ups);

  (void) smart_poll('c', ups);
  (void) smart_poll('c', ups);

  /* UPS_NAME */
  strcpy(ups->name, smart_poll('c', ups)); }

/*********************************************************************/
void setup_ups_sensitivity(UPSINFO *ups)
{ char response[32];
  char sensitivity[8];
  char a;
  response[0] = '\0';
  sensitivity[0] = '\0';

  strcpy(sensitivity, smart_poll('s', ups));
  while(strcmp(sensitivity, ups->sensitivity) != 0)
    { a = '-';
      write(ups->fd, &a, 1);
      getline(response, ups);
      if (strcmp(response, "NO") == 0)
        break;
      a = 's';
      write(ups->fd, &a, 1);
      getline(response, ups);
      strcpy(sensitivity, smart_poll('s', ups)); }

  /* SENSITIVITY */
  strcpy(ups->sensitivity, smart_poll('s', ups)); }

/*********************************************************************/
void setup_ups_wakeup(UPSINFO *ups)
{ char response[32];
  char a;
  response[0] = '\0';

  while(ups->wakeup != atof(smart_poll('r', ups)))
    { a = '-';
      write(ups->fd, &a, 1);
      getline(response, ups);
      if (strcmp(response, "NO") == 0)
        break;
      a = 'r';
      write(ups->fd, &a, 1);
      getline(response, ups); }

  /* WAKEUP_DELAY */
  ups->wakeup = atof(smart_poll('r', ups)); }

/*********************************************************************/
void setup_ups_sleep(UPSINFO *ups)
{ char response[32];
  char a;
  response[0] = '\0';

  while(ups->sleep != atof(smart_poll('p', ups)))
    { a = '-';
      write(ups->fd, &a, 1);
      getline(response, ups);
      if (strcmp(response, "NO") == 0)
        break;
      a = 'p';
      write(ups->fd, &a, 1);
      getline(response, ups); }

  /* SLEEP_DELAY */
  ups->sleep = atof(smart_poll('p', ups)); }

/*********************************************************************/
void setup_ups_lo_xfer(UPSINFO *ups)
{ char response[32];
  char a;
  int temp[4];
  int lopoint[5];
  int lo_temp, i;
  response[0] = '\0';

  for (i=0;i<5;i++)
    lopoint[i] = 0;

  lo_temp = 1000;
  for (i=0;i<4;i++)
    { temp[i] = atof(smart_poll('l', ups));
      if (temp[i] < lo_temp)
        lo_temp = temp[i];
      a = '-';
      write(ups->fd, &a, 1);
      getline(response, ups);
      if (strcmp(response, "NO") == 0)
        break;
      a = 'l';
      write(ups->fd, &a, 1);
      getline(response, ups); }

  while(lo_temp != atof(smart_poll('l', ups)))
    { a = '-';
      write(ups->fd, &a, 1);
      getline(response, ups);
      if (strcmp(response, "NO") == 0)
        break;
      a = 'l';
      write(ups->fd, &a, 1);
      getline(response, ups); }

  for (i=0;i<4;i++)
    { lopoint[i] = atof(smart_poll('l', ups));
      a = '-';
      write(ups->fd, &a, 1);
      getline(response, ups);
      if (strcmp(response, "NO") == 0)
        break;
      a = 'l';
      write(ups->fd, &a, 1);
      getline(response, ups); }

  while(lopoint[ups->lotrans] != atof(smart_poll('l', ups)))
    { a = '-';
      write(ups->fd, &a, 1);
      getline(response, ups);
      if (strcmp(response, "NO") == 0)
        break;
      a = 'l';
      write(ups->fd, &a, 1);
      getline(response, ups); }

  /* LOW_TRANSFER_LEVEL */
  ups->lotrans = atoi(smart_poll('l', ups)); }

/*********************************************************************/
void setup_ups_hi_xfer(UPSINFO *ups)
{ char response[32];
  char a;
  int temp[4];
  int hipoint[5];
  int lo_temp, i;
  response[0] = '\0';

  for (i=0;i<5;i++)
    hipoint[i] = 0;

  lo_temp = 1000;
  for (i=0;i<4;i++)
    { temp[i] = atof(smart_poll('u', ups));
      if (temp[i] < lo_temp)
        lo_temp = temp[i];
      a = '-';
      write(ups->fd, &a, 1);
      getline(response, ups);
      if (strcmp(response, "NO") == 0)
        break;
      a = 'u';
      write(ups->fd, &a, 1);
      getline(response, ups); }

  while(lo_temp != atof(smart_poll('u', ups)))
    { a = '-';
      write(ups->fd, &a, 1);
      getline(response, ups);
      if (strcmp(response, "NO") == 0)
        break;
      a = 'u';
      write(ups->fd, &a, 1);
      getline(response, ups); }

  for (i=0;i<4;i++)
    { a = '-';
      write(ups->fd, &a, 1);
      getline(response, ups);
      if (strcmp(response, "NO") == 0)
        break;
      a = 'u';
      write(ups->fd, &a, 1);
      getline(response, ups);
      hipoint[i] = atof(smart_poll('u', ups)); }

  while(hipoint[ups->hitrans] != atof(smart_poll('u', ups)))
    { a = '-';
      write(ups->fd, &a, 1);
      getline(response, ups);
      if (strcmp(response, "NO") == 0)
        break;
      a = 'u';
      write(ups->fd, &a, 1);
      getline(response, ups); }

  /* HIGH_TRANSFER_LEVEL */
  ups->hitrans = atoi(smart_poll('u', ups)); }

/*********************************************************************/
void setup_ups_chargepoint(UPSINFO *ups)
{ char response[32];
  char a;
  int temp[4];
  int chargepoint[5];
  int lo_temp, i;
  response[0] = '\0';

  for (i=0;i<5;i++)
    chargepoint[i] = 0;

  lo_temp = 1000;
  for (i=0;i<4;i++)
    { temp[i] = atof(smart_poll('e', ups));
      if (temp[i] < lo_temp)
        lo_temp = temp[i];
      a = '-';
      write(ups->fd, &a, 1);
      getline(response, ups);
      if (strcmp(response, "NO") == 0)
        break;
      a = 'e';
      write(ups->fd, &a, 1);
      getline(response, ups); }

  while(lo_temp != atof(smart_poll('e', ups)))
    { a = '-';
      write(ups->fd, &a, 1);
      getline(response, ups);
      if (strcmp(response, "NO") == 0)
        break;
      a = 'e';
      write(ups->fd, &a, 1);
      getline(response, ups); }

  for (i=0;i<4;i++)
    { a = '-';
      write(ups->fd, &a, 1);
      getline(response, ups);
      if (strcmp(response, "NO") == 0)
        break;
      a = 'e';
      write(ups->fd, &a, 1);
      getline(response, ups);
      chargepoint[i] = atof(smart_poll('e', ups)); }

  while(chargepoint[ups->minon] != atof(smart_poll('e', ups)))
    { a = '-';
      write(ups->fd, &a, 1);
      getline(response, ups);
      a = 'e';
      if (strcmp(response, "NO") == 0)
        break;
      write(ups->fd, &a, 1);
      getline(response, ups); }

  /* UPS_BATT_CAP_RETURN */
  ups->minon = atoi(smart_poll('e', ups)); }

/*********************************************************************/
void setup_ups_alarm(UPSINFO *ups)
{ char response[32];
  char beepstate[8];
  char a;
  response[0] = '\0';
  beepstate[0] = '\0';

  strcpy(beepstate, smart_poll('k', ups));
  while(strcmp(beepstate, ups->beepstate) != 0)
    { a = '-';
      write(ups->fd, &a, 1);
      getline(response, ups);
      if (strcmp(response, "NO") == 0)
        break;
      a = 'k';
      write(ups->fd, &a, 1);
      getline(response, ups);
      strcpy(beepstate, smart_poll('k', ups)); }

  /* ALARM_STATUS */
  strcpy(ups->beepstate, smart_poll('k', ups)); }

/*********************************************************************/
void setup_ups_lowbatt(UPSINFO *ups)
{ char response[32];
  char a;
  response[0] = '\0';

  while(ups->lowbatt != atof(smart_poll('q', ups)))
    { a = '-';
      write(ups->fd, &a, 1);
      getline(response, ups);
      if (strcmp(response, "NO") == 0)
        break;
      a = 'q';
      write(ups->fd, &a, 1);
      getline(response, ups); }

  /* LOWBATT_SHUTDOWN_LEVEL */
  ups->lowbatt = atof(smart_poll('q', ups)); }

/*********************************************************************/
void setup_ups_selftest(UPSINFO *ups)
#ifdef __SETUP_SELFTEST
{ char response[32];
  char a;
  response[0] = '\0';

  if (strcmp(response, "NO") == 0)
    break;

#else
{
#endif /* __SETUP_SELFTEST */
}

/*********************************************************************/
void setup_UPSlink_flags(UPSINFO *ups)
{ char response[32];
  response[0] = '\0';

  ups->UPSlink_flags = 0;

  /* UPS_STATUS */
  strcpy(response, smart_poll('Q', ups));
  if (strlen(response) && (strcmp(response,"NA") != 0))
    ups->UPSlink_flags |= 0x00000001;

  /* ONBATT_STATUS_FLAG */
  strcpy(response, smart_poll('9', ups));
  if (strlen(response) && (strcmp(response,"NA") != 0))
    ups->UPSlink_flags |= 0x00000002;

  /* POWER_FAIL_HISTORY */
  strcpy(response, smart_poll('G', ups));
  if (strlen(response) && (strcmp(response,"NA") != 0))
    ups->UPSlink_flags |= 0x00000004;

  /* UPS Self-Test Status and return on-line? */
  strcpy(response, smart_poll('X', ups));
  if (strlen(response) && (strcmp(response,"NA") != 0))
    ups->UPSlink_flags |= 0x00000008;

  /* LINE_VOLTAGE */
  strcpy(response, smart_poll('L', ups));
  if (strlen(response) && (strcmp(response,"NA") != 0))
    ups->UPSlink_flags |= 0x00000010;

  /* UPS_LINE_MAX */
  strcpy(response, smart_poll('M', ups));
  if (strlen(response) && (strcmp(response,"NA") != 0))
    ups->UPSlink_flags |= 0x00000020;

  /* UPS_LINE_MIN */
  strcpy(response, smart_poll('N', ups));
  if (strlen(response) && (strcmp(response,"NA") != 0))
    ups->UPSlink_flags |= 0x00000040;

  /* OUTPUT_VOLTAGE */
  strcpy(response, smart_poll('O', ups));
  if (strlen(response) && (strcmp(response,"NA") != 0))
    ups->UPSlink_flags |= 0x00000080;

  /* BATT_FULL */
  strcpy(response, smart_poll('f', ups));
  if (strlen(response) && (strcmp(response,"NA") != 0))
    ups->UPSlink_flags |= 0x00000100;

  /* BATT_VOLTAGE */
  strcpy(response, smart_poll('B', ups));
  if (strlen(response) && (strcmp(response,"NA") != 0))
    ups->UPSlink_flags |= 0x00000200;

  /* UPS_LOAD */
  strcpy(response, smart_poll('P', ups));
  if (strlen(response) && (strcmp(response,"NA") != 0))
    ups->UPSlink_flags |= 0x00000400;

  /* LINE_FREQ */
  strcpy(response, smart_poll('F', ups));
  if (strlen(response) && (strcmp(response,"NA") != 0))
    ups->UPSlink_flags |= 0x00000800;

  /* UPS_RUNTIME_LEFT */
  strcpy(response, smart_poll('j', ups));
  if (strlen(response) && (strcmp(response,"NA") != 0))
    ups->UPSlink_flags |= 0x00001000;

  /* UPS_TEMP */
  strcpy(response, smart_poll('C', ups));
  if (strlen(response) && (strcmp(response,"NA") != 0))
    ups->UPSlink_flags |= 0x00002000;

  /* DIP_SWITCH_SETTINGS */
  strcpy(response, smart_poll('7', ups));
  if (strlen(response) && (strcmp(response,"NA") != 0))
    ups->UPSlink_flags |= 0x00004000;

  /* UNKNOWN */
  strcpy(response, smart_poll('8', ups));
  if (strlen(response) && (strcmp(response,"NA") != 0))
    ups->UPSlink_flags |= 0x00008000;

  /* SENSITIVITY */
  strcpy(response, smart_poll('s', ups));
  if (strlen(response) && (strcmp(response,"NA") != 0))
    ups->UPSlink_flags |= 0x00010000;

  /* WAKEUP_DELAY */
  strcpy(response, smart_poll('r', ups));
  if (strlen(response) && (strcmp(response,"NA") != 0))
    ups->UPSlink_flags |= 0x00020000;

  /* SLEEP_DELAY */
  strcpy(response, smart_poll('p', ups));
  if (strlen(response) && (strcmp(response,"NA") != 0))
    ups->UPSlink_flags |= 0x00040000;

  /* LOW_TRANSFER_LEVEL */
  strcpy(response, smart_poll('l', ups));
  if (strlen(response) && (strcmp(response,"NA") != 0))
    ups->UPSlink_flags |= 0x00080000;

  /* HIGH_TRANSFER_LEVEL */
  strcpy(response, smart_poll('u', ups));
  if (strlen(response) && (strcmp(response,"NA") != 0))
    ups->UPSlink_flags |= 0x00100000;

  /* UPS_BATT_CAP_RETURN */
  strcpy(response, smart_poll('e', ups));
  if (strlen(response) && (strcmp(response,"NA") != 0))
    ups->UPSlink_flags |= 0x00200000;

  /* ALARM_STATUS */
  strcpy(response, smart_poll('k', ups));
  if (strlen(response) && (strcmp(response,"NA") != 0))
    ups->UPSlink_flags |= 0x00400000;

  /* LOWBATT_SHUTDOWN_LEVEL */
  strcpy(response, smart_poll('q', ups));
  if (strlen(response) && (strcmp(response,"NA") != 0))
    ups->UPSlink_flags |= 0x00800000;

  /* UPS_NAME */
  strcpy(response, smart_poll('c', ups));
  if (strlen(response) && (strcmp(response,"NA") != 0))
    ups->UPSlink_flags |= 0x01000000;

  /* UPS_SELFTEST */
  strcpy(response, smart_poll('E', ups));
  if (strlen(response) && (strcmp(response,"NA") != 0))
    ups->UPSlink_flags |= 0x02000000;
}

/*********************************************************************/
void setup_extended(UPSINFO *ups)
{ setup_UPSlink_flags(ups);

  setup_ups_model(ups);

#ifdef __FLAG_REPORTS
  fprintf(stderr, "\n%s: UPSlink_flags 0x%08lx", ups->argvalue, ups->UPSlink_flags);
#endif /* __FLAG_REPORTS */

#ifdef __DEBUG_NAME
  if (ups->UPSlink_flags & 0x01000000)
    setup_ups_name(ups);
#endif /* __DEBUG_NAME */

  if (ups->UPSlink_flags & 0x00010000)
    setup_ups_sensitivity(ups);

  if (ups->UPSlink_flags & 0x00020000)
    setup_ups_wakeup(ups);

  if (ups->UPSlink_flags & 0x00040000)
    setup_ups_sleep(ups);

  if (ups->UPSlink_flags & 0x00080000)
    setup_ups_lo_xfer(ups);

  if (ups->UPSlink_flags & 0x00100000)
    setup_ups_hi_xfer(ups);

  if (ups->UPSlink_flags & 0x00200000)
    setup_ups_chargepoint(ups);

  /* ALARM_STATUS */
  if (ups->UPSlink_flags & 0x00400000)
    setup_ups_alarm(ups);

  /* LOWBATT_SHUTDOWN_LEVEL */
  if (ups->UPSlink_flags & 0x00800000)
    setup_ups_lowbatt(ups);

  /* UPS_SELFTEST */
  if (ups->UPSlink_flags & 0x02000000)
    setup_ups_selftest(ups);
 }
