/*
 *  defines.h -- header file for apcupsd package
 *
 *  apcupsd.c -- Simple Daemon to catch power failure signals from a
 *               BackUPS, BackUPS Pro, or SmartUPS (from APCC).
 *            -- Now SmartMode support for SmartUPS and BackUPS Pro.
 *
 *  Copyright (C) 1996-98 Andre M. Hedrick <hedrick@astro.dyer.vanderbilt.edu>
 *                Vanderbilt University, Dyer Observatory.
 *  All rights reserved.
 *
 */

#ifndef _APC_DEFINES_H
#define _APC_DEFINES_H

#ifdef __freebsd__
#define APCCONF      "/usr/local/etc/apcupsd.conf"
#define UPS_STAT     "/var/log/apcupsd.status"
#define PWRFAIL      "/var/run/powerfail"
#endif /* __freebsd__ */

#ifndef APCCONF
#define APCCONF      "/etc/apcupsd.conf"
#endif

#ifndef UPS_STAT
#define UPS_STAT     "/etc/apcupsd.status"
#endif

#ifndef PWRFAIL
#define PWRFAIL      "/etc/powerfail"
#endif

#define NOLOGIN      "/etc/nologin"
#define APCPID       "/var/run/apcupsd.pid"
#define LOG_FILE     "/var/log/apcupsd.log"
#define APC_RE_NET   "/tmp/apcupsd.re-net"
#define LOCK_PREFIX  "/LCK.."

#define APC_MAGIC    "apcupsd-linux-4.0"
/* length of APC_MAGIC for convenience - includes NULL termination */
/* APC_MAGIC_SIZE is nice to define here, but */
/* power/include/apc_struct.h requires this.  Now there are */
/* interdependencies among the include files.  This can easily be */
/* solved with include magic or the sizes here can just be inserted */
/* *.h files and strlen() can be used in *.c files - although this */
/* just migrates the hard-coded number to a place away from where */
/* it makes sense.  I vote for doing the include magic . . . */
#define APC_MAGIC_SIZE 18
#define ACCESS_MAGIC "apcaccess-linux-4.0"
/* length of ACCESS_MAGIC for convenience - includes NULL termination */
/* ditto */
#define ACCESS_MAGIC_SIZE 20

#define POWERFAIL    "POWER FAILURE"
#define REDO_NET     "RESET NET"

#define ACCESS_FIFO  "/tmp/apcaccess_fifo"
#define PIPE_FIFO    "/tmp/apcpipe_fifo"

#define MAXSTRING     256
#define MESSAGELEN    256
#define MAXTOKENLEN   80
#define MAXSLAVES     20

#define DEFAULT_SPEED B2400

#define GO_ON_BATT   'W'
#define GO_ON_LINE   'X'
#define LIGHTS_TEST  'A'
#define FAILURE_TEST 'U'

/*
 * Future additions for contolled discharing of batteries
 * extend lifetimes.
 */

#define DISCHARGE    'D'
#define CHARGE_LIM   25

#define UPS_ENABLED  '?'
#define UPS_ON_BATT  '!'
#define UPS_ON_LINE  '$'
#define BATT_LOW     '%'
#define BATT_OK      '+'
#define UPS_GARBAGE  '|'
#define UPS_TRAILOR  ':'
#define UPS_LF       '\n'
#define UPS_CR       '\r'

/* These seem unavoidable :-( */
#ifndef TRUE
#  define TRUE            1
#endif
#ifndef FALSE
#  define FALSE           0
#endif

#endif /* _APC_DEFINES_H */
