/*
 *  extern.h  -- header file for apcupsd package
 *
 *  apcupsd.c -- Simple Daemon to catch power failure signals from a
 *               BackUPS, BackUPS Pro, or SmartUPS (from APCC).
 *            -- Now SmartMode support for SmartUPS and BackUPS Pro.
 *
 *  Copyright (C) 1996-98 Andre M. Hedrick <hedrick@astro.dyer.vanderbilt.edu>
 *                Vanderbilt University, Dyer Observatory.
 *  All rights reserved.
 *
 */

#ifndef _APC_EXTERN_H
#define _APC_EXTERN_H

/*********************************************************************/
/* Function ProtoTypes                                               */
/*********************************************************************/

extern UPSINFO myUPS;

/*
 * In apcupsd.c
 */
extern void powerfail(int ok);
extern void logonfail(int ok);
extern int make_file(const char *s);
extern int remove_file(const char *s);

extern void terminate(int sig);
extern void clear_files(void);
extern void make_pid(void);

extern int setup_serial(UPSINFO *ups);
extern int create_lockfile(UPSINFO *ups);                     /* BSC */
extern void delete_lockfile(UPSINFO *ups, int boolean);       /* BSC */
extern void rethink_lockfile(UPSINFO *ups);                   /* BSC */
extern void kill_serial(UPSINFO *ups, int killpwr);
extern int check_serial(UPSINFO *ups);
extern void do_serial(UPSINFO *ups);

/*
 * In apcnet.c
 */
extern int reconnect_master(UPSINFO *ups, int who);
extern int prepare_master(UPSINFO *ups);
extern int send_to_slaves(UPSINFO *ups);
extern int send_to_each(UPSINFO *ups, int who);

extern int reconnect_slave(UPSINFO *ups);
extern int prepare_slave(UPSINFO *ups);
extern int get_from_master(UPSINFO *ups);
extern void kill_net(UPSINFO *ups, int killpwr);
extern void do_net(UPSINFO *ups);

/*
 * In apcpipe.c
 */
extern int pipe_requests(UPSINFO *ups);
extern int pipe_reconfig(UPSINFO *ups, CONFIGINFO *config);
extern int pipe_master_status(UPSINFO *ups);
extern int pipe_call_shutdown(UPSINFO *ups);
extern int pipe_slave_reconnect(UPSINFO *ups);
extern int pipe_slave_release(UPSINFO *ups);

/*
 * In apcconfig.c
 */
extern void zero_ups_struct(UPSINFO *ups);
extern int check_for_config(UPSINFO *ups);

/*
 * In apcsetup.c
 */
extern char *setup_ups_model_V_codes(char *s, UPSINFO *ups);
extern char *setup_ups_model_b_codes(char *s, UPSINFO *ups);
extern void setup_ups_model(UPSINFO *ups);
extern void setup_ups_name(UPSINFO *ups);
extern void setup_ups_sensitivity(UPSINFO *ups);
extern void setup_ups_wakeup(UPSINFO *ups);
extern void setup_ups_sleep(UPSINFO *ups);
extern void setup_ups_lo_xfer(UPSINFO *ups);
extern void setup_ups_hi_xfer(UPSINFO *ups);
extern void setup_ups_chargepoint(UPSINFO *ups);
extern void setup_ups_alarm(UPSINFO *ups);
extern void setup_ups_lowbatt(UPSINFO *ups);
extern void setup_ups_selftest(UPSINFO *ups);
extern void setup_UPSlink_flags(UPSINFO *ups);
extern void setup_extended(UPSINFO *ups);

/*
 * In apcreports.c
 */
extern int updating_procfs(FILE *file, UPSINFO *ups);
extern int updating_logs(FILE *file, UPSINFO *ups);
extern int logprintf(char *fmt, ...);

/*
 * In apcsmart.c
 */
extern void sig_alarm(int sig);
extern int getline(char *s, UPSINFO *ups);
extern void UPSlinkCheck(UPSINFO *ups);
extern char *smart_poll(char cmd, UPSINFO *ups);
extern int fillUPS(UPSINFO *ups);
extern void test_status_bits(UPSINFO *ups);

#endif /* _APC_EXTERN_H */
