/*
 *  struct.h  -- header file for apcupsd package
 *
 *  apcupsd.c -- Simple Daemon to catch power failure signals from a
 *               BackUPS, BackUPS Pro, or SmartUPS (from APCC).
 *            -- Now SmartMode support for SmartUPS and BackUPS Pro.
 *
 *  Copyright (C) 1996-98 Andre M. Hedrick <hedrick@astro.dyer.vanderbilt.edu>
 *                Vanderbilt University, Dyer Observatory.
 *  All rights reserved.
 *
 */

/*********************************************************************/

#ifndef _APC_STRUCT_H
#define _APC_STRUCT_H

typedef enum {
 NO_CABLE=0,     /* Default Disable            */
 CUSTOM_SIMPLE,  /* SIMPLE cable simple        */
 APC_940_0020B,  /* APC cable number 940-0020B */
 APC_940_0023A,  /* APC cable number 940-0023A */
 CUSTOM_SMART,   /* SMART cable smart          */
 APC_940_0024B,  /* APC cable number 940-0024B */
 APC_940_0024C,  /* APC cable number 940-0024C */
 APC_940_0095A,  /* APC cable number 940-0095A */
 APC_940_0095C,  /* APC cable number 940-0095C */
 APC_NET,        /* Ethernet Link              */
 APC_940_00XXX   /* APC cable number UNKNOWN   */
} UpsCable;

/* The order of these UpsModes is important!! */
typedef enum {
 NO_UPS=0,   /* Default Disable      */
 BK,         /* Simple Signal        */
 SHAREBASIC, /* Simple Signal, Share */
 NETUPS,     /*                      */
 BKPRO,      /* SubSet Smart Signal  */
 VS,         /* SubSet Smart Signal  */
 NBKPRO,     /* Smarter BKPRO Signal */
 SMART,      /* Smart Signal         */
 MATRIX,     /* Smart Signal         */
 SHARESMART  /* Smart Signal, Share  */
} UpsMode;

typedef enum {
  NO_CLASS=0,
  STANDALONE,     /**/
  SHARESLAVE,     /**/
  NETSLAVE,       /**/
  SHAREMASTER,    /**/
  NETMASTER,      /**/
  SHARENETMASTER  /**/
} ClassMode;

typedef enum {
 NO_SHARE_NET=0,
 DISABLE,    /* Disable Share or Net UPS  */
 SHARE,      /* ShareUPS Internal         */
 NET,        /* NetUPS                    */
 SHARENET    /* Share and Net, Master     */
} ShareNetMode;

typedef enum {
 NO_LOGON=0,   
 NEVER,      /* Disable Setting NoLogon               */
 TIMEOUT,    /* Based on TIMEOUT + 10 percent         */
 PERCENT,    /* Based on PERCENT + 10 percent         */
 MINUTES,    /* Based on MINUTES + 10 percent         */
 ALWAYS      /* Stop All New Login Attempts. but ROOT */
} NoLoginMode;

typedef struct {
	char *name;
	char *long_name;
	int type;
} GENINFO; /* for static declaration of data */

typedef struct {
	char name[MAXSTRING];
	char long_name[MAXSTRING];
	int type;
} INTERNALGENINFO; /* for assigning into upsinfo */

typedef struct {
	char name[MAXSTRING];
	int present;
	char magic[MAXSTRING];
	int socket;
	struct sockaddr_in addr;
} SLAVEINFO;

typedef	struct {
  int fd;
  char argvalue[MAXSTRING];

  unsigned long UPSlink_flags;

  int ShutDown;
  int BatteryUp;

  INTERNALGENINFO cable;
  INTERNALGENINFO enable_access;
  INTERNALGENINFO nologin;
  INTERNALGENINFO mode;
  INTERNALGENINFO class;
  INTERNALGENINFO sharenet;

  int LineUp;
  int OldLineUp;
  int BattUp;
  int OldBattUp;
  int LineDown;
  int OldLineDown;

  int LineLevel;

  char nine[8];
  char eight[8];
  unsigned int seven;
  char G[8];
  char X[8];
  double BatLoad;
  double LineMin;
  double LineMax;
  double UPSLoad;
  double LineFreq;
  double LineVoltage;
  double OutputVoltage;
  double UPSTemp;
  double BattVoltage;
  unsigned int Status;
  double TimeLeft;

  int annoy;
  int maxtime;
  int delay;
  int nologin_time;
  int nologin_file;
  int proctime;
  int logtime;
  int nettime;
  int percent;
  int runtime;

  char sensitivity[8];
  int wakeup;
  int sleep;

  int lotrans;
  int hitrans;
  int minon;
  int lowbatt;

  char beepstate[8];
  char name[MAXSTRING];

  char device[MAXSTRING];          /**/
  char control_path[MAXSTRING];    /**/
  char powerout[MAXSTRING];        /* Warning, There is Power problems.      */
  char onbatt[MAXSTRING];          /* Warning, System running on UPS.        */
  char failing[MAXSTRING];         /* Battery Power Failing, Normal Shutdown.*/
  char timeout[MAXSTRING];         /* Online Battery Timeout, Normal Shutdown.*/
  char loadlimit[MAXSTRING];       /* Online Battery Limit, Normal Shutdown. */
  char runlimit[MAXSTRING];        /* Normal Shutdown. */
  char doshutdown[MAXSTRING];      /* Perform Normal Shutdown.               */
  char mainsback[MAXSTRING];       /* Power is back up.                      */
  char waitasec[MAXSTRING];        /* Attempting to halt shutdown.           */
  char annoyme[MAXSTRING];         /* Power Problems, Log Off System Now!!!! */
  char emergency[MAXSTRING];       /* Emergency, Possible Battery Failure!!!!*/
  char changeme[MAXSTRING];        /* Emergency, Batteries Have Failed!!!!   */
  char restartme[MAXSTRING];       /* Emergency, Restart UPS gone to long!!! */
  char remote_shutdown[MAXSTRING]; /* Remote Call Shutdown, Normal Shutdown. */

  char batt_cmd[FILENAME_MAX];     /**/
  char time_cmd[FILENAME_MAX];     /**/
  char load_cmd[FILENAME_MAX];     /**/
  char limit_cmd[FILENAME_MAX];    /**/
  char pwr_cmd[FILENAME_MAX];      /**/
  char ret_cmd[FILENAME_MAX];      /**/
  char remote_cmd[FILENAME_MAX];   /**/

  int NetUpsPort;                  /* Our communication port */
  char master_name[FILENAME_MAX];  /**/
  SLAVEINFO slaves[MAXSLAVES];     /* Slaves description */
  int slave_count;                 /**/

  char lockpath[FILENAME_MAX];     /* BSC, made static -RF */
  int lockfile;                    /* BSC */

  char usermagic[MAXSTRING];       /* More Security */

  int load;                        /**/
  int timedout;                    /**/
  int timelout;                    /**/
  int emergencydown;               /**/
  int remotedown;                  /**/

  int remote_state;                /**/
} UPSINFO;

typedef struct datainfo {
  char apcmagic[APC_MAGIC_SIZE];
  int update_master_config;
  int get_master_status;
  int slave_status;
  int call_master_shutdown;
  char accessmagic[ACCESS_MAGIC_SIZE];
} DATAINFO;

typedef struct configinfo {
  int new_annoy;
  int new_maxtime;
  int new_delay;
#ifdef __NOLOGIN
  int new_nologin;
#endif /* __NOLOGIN */
  int new_proctime;
  int new_logtime;
  int new_nettime;
  int new_percent;

  char new_batt_cmd[FILENAME_MAX];    /**/
  char new_time_cmd[FILENAME_MAX];    /**/
  char new_load_cmd[FILENAME_MAX];    /**/
  char new_pwr_cmd[FILENAME_MAX];     /**/
  char new_ret_cmd[FILENAME_MAX];     /**/
  char new_remote_cmd[FILENAME_MAX];  /**/
} CONFIGINFO;

/* number of magic strings in netdata structure */
#define NUM_MAGIC 2
/* number of ints in netdata structure */
#define NUM_INT 13

typedef struct netdata {
  char apcmagic[APC_MAGIC_SIZE];
  char usermagic[APC_MAGIC_SIZE];
  int status;
  int lowbatt;
  int battery;
  int shutdown;
  int load;
  int timeout;
  int runtime;
  int nologin;
  int emergency;
  int remote;
  int order;
  int check_time;
  int release_me;
} netdata;

#endif /* _APC_STRUCT_H */
